@@ -4,6 +4,7 @@ import os
 import shlex
 from dataclasses import dataclass
 from typing import List, NamedTuple
+from pyspark import __version__ as spark_version
 
 from spyt.dependency_utils import require_yt_client
 
@@ -15,6 +16,7 @@ from yt.wrapper.http_helpers import get_token, get_user_name  # noqa: E402
 from yt.wrapper.spec_builders import VanillaSpecBuilder  # noqa: E402
 
 from .conf import ytserver_proxy_attributes, get_spark_distributive  # noqa: E402
+from .conf import read_metrics_conf, read_spark_defaults_conf  # noqa: E402
 from .utils import SparkDiscovery, call_get_proxy_address_url, parse_memory  # noqa: E402
 from .enabler import SpytEnablers  # noqa: E402
 from .version import __version__  # noqa: E402
@@ -47,9 +49,7 @@ class SparkDefaultArguments(object):
         return {
             "operation_spec": {
                 "annotations": {
-                    "is_spark": True,
-                    "solomon_resolver_tag": "spark",
-                    "solomon_resolver_ports": [27100],
+                    "is_spark": True
                 },
                 "max_failed_job_count": 10000,
                 "max_stderr_count": 150,
@@ -73,6 +73,7 @@ class CommonComponentConfig():
     cluster_log_level: str = "INFO"
     rpc_job_proxy: bool = False
     rpc_job_proxy_thread_pool_size: int = 4
+    enable_ytsaurus_shuffle: bool = False
     tcp_proxy_range_start: int = 30000
     tcp_proxy_range_size: int = 100
     enable_stderr_table: bool = False
@@ -196,6 +197,22 @@ def _launcher_command(component: str, common_params: CommonSpecParams, additiona
     return " && ".join(commands)
 
 
+def _create_file_and_layer_paths(config, enable_squashfs: bool, spark_distributive_path):
+    file_paths = [path for path in config["file_paths"] if
+                  not enable_squashfs or
+                  not path.endswith('spyt-package.zip')]
+    layer_paths = []
+
+    if enable_squashfs:
+        layer_paths.append(f"{config['spark_yt_base_path']}/spyt-package.squashfs")
+        layer_paths.append(spark_distributive_path)
+    else:
+        file_paths.append(spark_distributive_path)
+
+    layer_paths += config["squashfs_layer_paths" if enable_squashfs else "layer_paths"]
+    return file_paths, layer_paths
+
+
 def build_livy_spec(builder: VanillaSpecBuilder, common_params: CommonSpecParams, config: LivyConfig):
     livy_launcher_opts = [
         f"--driver-cores {config.livy_driver_cores}",
@@ -286,13 +303,15 @@ def build_master_spec(builder: VanillaSpecBuilder, common_params: CommonSpecPara
     master_environment = update(common_params.environment, master_environment)
     master_task_spec = copy.deepcopy(common_params.task_spec)
     master_task_spec["environment"] = master_environment
-    builder.begin_task("master") \
+    task = builder.begin_task("master") \
         .job_count(1) \
         .command(master_command) \
         .memory_limit(parse_memory(config.master_memory_limit)) \
         .cpu_limit(2) \
-        .spec(master_task_spec) \
-        .end_task()
+        .spec(master_task_spec)
+    if common_params.spark_conf.get("spark.ytsaurus.use_assigned_ports.master") == "true":
+        task.port_count(2)
+    task.end_task()
 
 
 def build_worker_spec(builder: VanillaSpecBuilder, job_type: str, ytserver_proxy_path: str, tvm_enabled: bool,
@@ -363,6 +382,7 @@ def build_worker_spec(builder: VanillaSpecBuilder, job_type: str, ytserver_proxy
     worker_task_spec = copy.deepcopy(common_params.task_spec)
     worker_task_spec["environment"] = worker_environment
     worker_task_spec["rpc_proxy_worker_thread_pool_size"] = common_params.config.rpc_job_proxy_thread_pool_size
+    worker_task_spec["enable_shuffle_service_in_job_proxy"] = common_params.config.enable_ytsaurus_shuffle
     worker_ram_memory = parse_memory(config.res.memory)
     worker_local_dirs = "."
     if config.worker_disk_limit:
@@ -388,14 +408,16 @@ def build_worker_spec(builder: VanillaSpecBuilder, job_type: str, ytserver_proxy
         driver_environment["SPARK_DRIVER_RESOURCE"] = str(config.res.cores)
 
     spec = driver_task_spec if "drivers" == job_type else worker_task_spec
-    builder.begin_task(job_type) \
+    task = builder.begin_task(job_type) \
         .job_count(config.res.num) \
         .command(worker_command) \
         .memory_limit(worker_ram_memory + parse_memory(config.res.memory_overhead)) \
         .cpu_limit(config.res.cores + worker_cores_overhead) \
         .spec(spec) \
-        .file_paths(worker_file_paths) \
-        .end_task()
+        .file_paths(worker_file_paths)
+    if common_params.spark_conf.get("spark.ytsaurus.use_assigned_ports.worker") == "true":
+        task.port_count(2)
+    task.end_task()
 
 
 def build_spark_operation_spec(config: dict, client: YtClient,
@@ -460,7 +482,7 @@ def build_spark_operation_spec(config: dict, client: YtClient,
     environment["SPYT_HOME"] = common_config.spyt_home
     environment["SPARK_CLUSTER_VERSION"] = config["cluster_version"]  # TODO Rename to SPYT_CLUSTER_VERSION
     environment["SPYT_CLUSTER_VERSION"] = config["cluster_version"]
-    environment["SPARK_YT_SOLOMON_ENABLED"] = str(common_config.enablers.enable_solomon_agent)
+    environment["SPARK_YT_METRICS_ENABLED"] = str(common_config.enablers.enable_yt_metrics)
     environment["SPARK_YT_IPV6_PREFERENCE_ENABLED"] = \
         str(common_config.enablers.enable_preference_ipv6)  # COMPAT(alex-shishkin)
     environment["SPARK_YT_TCP_PROXY_ENABLED"] = str(common_config.enablers.enable_tcp_proxy)
@@ -468,24 +490,26 @@ def build_spark_operation_spec(config: dict, client: YtClient,
     environment["SPARK_YT_RPC_JOB_PROXY_ENABLED"] = str(common_config.rpc_job_proxy)
     if common_config.enablers.enable_byop:
         environment["SPARK_YT_BYOP_PORT"] = "27002"
-    if common_config.enablers.enable_solomon_agent:
-        environment["SOLOMON_PUSH_PORT"] = "27099"
+    if common_config.enablers.enable_yt_metrics:
+        metrics_conf = read_metrics_conf()
+        spark_defaults_conf = read_spark_defaults_conf()
+
+        spark_push_port = metrics_conf["*.sink.solomon.solomon_port"]
+        metrics_pull_port = spark_defaults_conf["spark.ytsaurus.metrics.pull.port"]
+        agent_pull_port = spark_defaults_conf["spark.ytsaurus.metrics.agent.pull.port"]
+        environment["YT_METRICS_SPARK_PUSH_PORT"] = spark_push_port
+        environment["YT_AGENT_METRICS_PULL_PORT"] = agent_pull_port
+        environment["YT_METRICS_PULL_PORT"] = metrics_pull_port
+        operation_spec["annotations"]["solomon_resolver_tag"] = "spark"
+        operation_spec["annotations"]["solomon_resolver_ports"] = [int(metrics_pull_port), int(agent_pull_port)]
+
     if common_config.enablers.enable_tcp_proxy:
         environment["SPARK_YT_TCP_PROXY_RANGE_START"] = str(common_config.tcp_proxy_range_start)
         environment["SPARK_YT_TCP_PROXY_RANGE_SIZE"] = str(common_config.tcp_proxy_range_size)
 
-    file_paths = [path for path in config["file_paths"] if
-                  not common_config.enablers.enable_squashfs or
-                  not path.endswith('spyt-package.zip')]
-    layer_paths = []
-
-    if common_config.enablers.enable_squashfs:
-        layer_paths.append(f"{config['spark_yt_base_path']}/spyt-package.squashfs")
-        layer_paths.append(spark_distributive_path)
-    else:
-        file_paths.append(spark_distributive_path)
-
-    layer_paths += config["squashfs_layer_paths" if common_config.enablers.enable_squashfs else "layer_paths"]
+    file_paths, layer_paths = _create_file_and_layer_paths(config,
+                                                           common_config.enablers.enable_squashfs,
+                                                           spark_distributive_path)
 
     common_task_spec = {
         "restart_completed_jobs": True,
@@ -530,3 +554,69 @@ def build_spark_operation_spec(config: dict, client: YtClient,
     return builder \
         .secure_vault(secure_vault) \
         .spec(operation_spec)
+
+
+def build_spark_connect_server_spec(client: YtClient, config, spark_conf: dict, enablers: SpytEnablers, java_home: str,
+                                    driver_memory: str, num_executors: int, executor_cores: int, executor_memory: str,
+                                    prefer_ipv6: bool, pool: str, grpc_port_start: int):
+    component_config = CommonComponentConfig(enable_tmpfs=False, rpc_job_proxy=True, enablers=enablers)
+
+    spark_distributive, spark_distributive_path = get_spark_distributive(client, enablers.enable_squashfs)
+    setup = setup_spyt_env(component_config.container_home, spark_distributive, enablers.enable_squashfs, [])
+    user = get_user_name(client=client)
+    yt_proxy = call_get_proxy_address_url(required=True, client=client)
+    spark_connect_jar = f"spark-connect_2.12-{spark_version}.jar"
+    network_project = spark_conf.get("spark.ytsaurus.network.project")
+    command = [
+        f"{component_config.spark_home}/bin/spark-submit",
+        f"--master ytsaurus://{yt_proxy}",
+        "--deploy-mode client",
+        "--class org.apache.spark.sql.connect.ytsaurus.SpytConnectServer",
+        f"--num-executors {num_executors}",
+        f"--executor-cores {executor_cores}",
+        f"--executor-memory {executor_memory}",
+        f"--queue {pool or user}",
+        f'--name "Spark connect driver for {user}"',
+        f"--conf spark.driver.extraJavaOptions='-Djava.net.preferIPv6Addresses={prefer_ipv6}'",
+        f"--conf spark.connect.grpc.binding.port={grpc_port_start}",
+        f"--jars {spark_connect_jar}",
+    ] + [f"--conf {key}={spark_conf[key]}" for key in spark_conf] + ["spark-internal"]
+
+    operation_spec = {
+        "title": "Spark connect server"
+    }
+
+    secure_vault = {
+        "YT_TOKEN": get_token(client=client)
+    }
+
+    file_paths, layer_paths = _create_file_and_layer_paths(config, enablers.enable_squashfs, spark_distributive_path)
+
+    file_paths.append(f"//home/spark/distrib/{spark_version.replace('.', '/')}/spark-connect_2.12-{spark_version}.jar")
+
+    environment = copy.deepcopy(config["environment"])
+    environment["JAVA_HOME"] = java_home
+    environment["SPARK_CONF_DIR"] = f"{component_config.spyt_home}/conf".replace("$HOME/", "")
+    environment["SPARK_CONNECT_CLASSPATH"] = f"{spark_connect_jar}"
+
+    task_spec = {
+        "layer_paths": layer_paths,
+        "file_paths": file_paths,
+        "enable_rpc_proxy_in_job_proxy": component_config.rpc_job_proxy,
+        "environment": environment,
+    }
+
+    if network_project:
+        task_spec["network_project"] = network_project
+
+    builder = VanillaSpecBuilder()
+    builder.begin_task("spark_connect_driver") \
+        .command(" ".join([setup, "&&"] + command)) \
+        .job_count(1) \
+        .cpu_limit(1) \
+        .memory_limit(parse_memory(driver_memory)) \
+        .spec(task_spec) \
+        .end_task()
+
+    return builder.secure_vault(secure_vault) \
+        .spec(operation_spec)
